#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <usbdrv.h>
#include "keycodes.h"
#include "timer.h"
#include "pt/pt.h"

#define SWITCH_PRESSED_1() ((PINC & (1<<PC1)) == 0)
#define SWITCH_PRESSED_2() ((PINC & (1<<PC2)) == 0)
#define SWITCH_PRESSED_3() ((PINC & (1<<PC3)) == 0)
#define SWITCH_PRESSED_4() ((PINC & (1<<PC4)) == 0)
#define SWITCH_PRESSED_0() ((PINC & (1<<PC0)) == 0)

#define RQ_GET_SWITCH_0         0
#define RQ_GET_SWITCH_1         1
#define RQ_GET_SWITCH_2         2
#define RQ_GET_SWITCH_3         3
#define RQ_GET_SWITCH_4         4

static uint8_t switch_state0;
static uint8_t switch_state1; 
static uint8_t switch_state2; 
static uint8_t switch_state3; 
static uint8_t switch_state4;   
// มาโครสำหรับจำลองการหน่วงเวลาใน protothread
#define PT_DELAY(pt,ms,tsVar) \
  tsVar = timer_millis(); \
  PT_WAIT_UNTIL(pt, timer_millis()-tsVar >= (ms));

/* ------------------------------------------------------------------------- */
typedef struct 
{
	uint8_t modifiers;  // แป้น Shift, Ctrl, Alt
	uint8_t keycode;   // แป้นทั่วไป
} keyboard_report_t;

keyboard_report_t reportBuffer; /* buffer for HID reports */
uchar idleRate = 10;            /* in 4 ms units */
struct pt taskSendKeys_pt;

/* ------------------------------------------------------------------------- */

PROGMEM const char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = 
{ /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
/* We use a simplifed keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */


void init_peri()
{
    // PC0..PC2 -> input
    DDRC &= ~((1<<PC2)|(1<<PC1)|(1<<PC0));

    // PC3,PC4 -> input
    DDRC &= ~((1<<PC3)|(1<<PC4));

    PORTC |= (1<<PC0);
    PORTC |= (1<<PC1);
    PORTC |= (1<<PC2);
    PORTC |= (1<<PC3);
    PORTC |= (1<<PC4);

}

//////////////////////////////////////////////////////////////
uchar usbFunctionSetup(uchar data[8])
{
 usbRequest_t *rq = (void *)data;

    if (rq->bRequest == RQ_GET_SWITCH_0 )
    {
        
        switch_state0 = SWITCH_PRESSED_0();
	usbMsgPtr = &switch_state0;
       
        return 1;
    }

    else if (rq->bRequest == RQ_GET_SWITCH_1)
    {
        switch_state1 = SWITCH_PRESSED_1();
        usbMsgPtr = &switch_state1;
       
        return 1;
}
    else if (rq->bRequest == RQ_GET_SWITCH_2)
    {
    
        switch_state2 = SWITCH_PRESSED_2();
	usbMsgPtr = &switch_state2;
       
        return 1;
}
   else if(rq->bRequest == RQ_GET_SWITCH_3) 
            {
         
        switch_state3 = SWITCH_PRESSED_3();
	usbMsgPtr = &switch_state3;
       
        return 1;
            }
    else if(rq->bRequest == RQ_GET_SWITCH_4) 
            {
         
        switch_state4 = SWITCH_PRESSED_4();
	usbMsgPtr = &switch_state4;
       
        return 1;
            }
	return 0;
}

//////////////////////////////////////////////////////////////
void sendKey(uint8_t keycode, uint8_t modifiers)
{
    reportBuffer.keycode = keycode;
    reportBuffer.modifiers = modifiers;
    usbSetInterrupt((uchar*)&reportBuffer, sizeof(reportBuffer));
}

//////////////////////////////////////////////////////////////
PT_THREAD(taskSendKeys(struct pt *pt))
{
    static uint32_t ts = 0;
    static uint8_t i;
    
    static uint8_t sequence[]  = {KEY_W};
    static uint8_t sequence1[] = {KEY_S};
    static uint8_t sequence2[] = {KEY_A};
    static uint8_t sequence3[] = {KEY_D};
    static uint8_t sequence4[] = {KEY_L};

    PT_BEGIN(pt);

    for (;;)
    {
        if ((PINC & (1<<PC0)) == 0)
       { for (int i = 0; i < sizeof(sequence); i++)
        {
            // Press a shift key for the first key in the sequence
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            if (i == 0)
                sendKey(sequence[i],KEY_MOD_LEFT_SHIFT);
            else
                sendKey(sequence[i],0);

            PT_DELAY(pt,10,ts);

            // Release key
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            sendKey(KEY_NONE,0);
            PT_DELAY(pt,10,ts);
        }
}
       else if  ((PINC & (1<<PC1)) == 0)
{
 for (int i = 0; i < sizeof(sequence); i++)
        {
            // Press a shift key for the first key in the sequence
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            if (i == 0)
                sendKey(sequence1[i],KEY_MOD_LEFT_SHIFT);
            else
                sendKey(sequence1[i],0);

            PT_DELAY(pt,10,ts);

            // Release key
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            sendKey(KEY_NONE,0);
            PT_DELAY(pt,10,ts);
        }
}
else if  ((PINC & (1<<PC2)) == 0)
{
 for (int i = 0; i < sizeof(sequence); i++)
        {
            // Press a shift key for the first key in the sequence
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            if (i == 0)
                sendKey(sequence2[i],KEY_MOD_LEFT_SHIFT);
            else
                sendKey(sequence2[i],0);

            PT_DELAY(pt,10,ts);

            // Release key
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            sendKey(KEY_NONE,0);
            PT_DELAY(pt,10,ts);
        }
}
else if  ((PINC & (1<<PC3)) == 0)
{
 for (int i = 0; i < sizeof(sequence); i++)
        {
            // Press a shift key for the first key in the sequence
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            if (i == 0)
                sendKey(sequence3[i],KEY_MOD_LEFT_SHIFT);
            else
                sendKey(sequence3[i],0);

            PT_DELAY(pt,10,ts);

            // Release key
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            sendKey(KEY_NONE,0);
            PT_DELAY(pt,10,ts);
        }
}
else if  ((PINC & (1<<PC4)) == 0)
{
 for (int i = 0; i < sizeof(sequence); i++)
        {
            // Press a shift key for the first key in the sequence
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            if (i == 0)
                sendKey(sequence4[i],KEY_MOD_LEFT_SHIFT);
            else
                sendKey(sequence4[i],0);

            PT_DELAY(pt,10,ts);

            // Release key
            PT_WAIT_UNTIL(pt,usbInterruptIsReady());
            sendKey(KEY_NONE,0);
            PT_DELAY(pt,10,ts);
        }
}
    }

    PT_END(pt);
}

//////////////////////////////////////////////////////////////
int main(void)
{
    init_peri();

    timer_init();
    usbInit();
    PT_INIT(&taskSendKeys_pt);

    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
        taskSendKeys(&taskSendKeys_pt);
    }

    return 0;
}
